/*=============================================================================================================
                                                   colony.h
===============================================================================================================

 Definition of the class Colony
 
 C++-code accompanying: 
 
        authors : Boris H. Kramer, G. Sander van Doorn, Babak M. S. Arani, Ido Pen
        title : Eusociality and the evolution of aging in superorganisms
 

 Written by:
        Boris H. Kramer and G. Sander van Doorn
        Groningen Institute for Evolutionary Life Sciences (Gelifes)
        University of Groningen
        the Netherlands
 
 Program version
        01/03/2019	

 =============================================================================================================*/

#ifndef colony_h
#define colony_h

//dependencies
#include "individual.h"
#include <vector>

 //define the class COLONY
class Colony { 
public:
	Colony(Queen * const);// constructor 
	~Colony(); //destructor
	Queen const * getQueen() const { return queen; } 
	void reproduce();//function prototype
	void survival();//function prototype
	void addWorkers();//function prototype
	size_t size() const { return workers.size() + (queen != nullptr); }// returns colony size including the queen  
	size_t nrOffspring() const { return offspring.size(); } //returns number of brood items
	bool isQueenless() const { return queen == nullptr; }//queenright or queenless 
	Female *getGyne();//function prototype
	void getData(size_t&, size_t&, size_t&, size_t&, std::vector<double>&, std::vector<double>&) const; 
	static const Male& getDrone(); 
	static void clearDrones() { drones.clear(); }
	double getFitness() const { return fitness; } // function to return fitness of the colony  ; access: population[i]->getFitness()
private:
	double fitness;
	std::vector<Female*> workers;// vector for each colony that stores the workers
	Queen* queen; //???
	std::vector<Female*> offspring;// vector for each colony that stores the brood
	static std::vector<Male> drones;//  one static vector vector for all members of the class colony to store the males
};

#endif