/*=============================================================================================================
                                                   individual.h
===============================================================================================================

 Definition of the class Indidivual
 
 C++-code accompanying: 
 
        authors : Boris H. Kramer, G. Sander van Doorn, Babak M. S. Arani, Ido Pen
        title : Eusociality and the evolution of aging in superorganisms
 

 Written by:
        Boris H. Kramer and G. Sander van Doorn
        Groningen Institute for Evolutionary Life Sciences (Gelifes)
        University of Groningen
        the Netherlands
 
 Program version
        01/03/2019	

 
 =============================================================================================================*/

#ifndef individual_h
#define individual_h

//external dependencies
#include<array>
#include<vector>
#include "parameters.h"
#include "corand.h"

//definition of the class gene
class Gene {
public:
	Gene() : dx(K, 0.0) {} // constructor for the start of the simulation 
	Gene(const Gene&) = default; // alternative constructor for subsequent queen generations 
	void mutate() { dx += geneticArchitecture->operator()(); }  //mutation function
	double operator()(const int &cs, const int &tr, const int &ag) { return dx[ag + (tr + cs * nTrait) * ageMax]; } 
	void getData(std::vector<double>&, std::vector<double>&) const; // // Member functions declaration;  output life history
	static void initGeneticArchitecture();// static member function or method with initial genetic architecture
private:
	Vector dx;
	static rnd::MultiNormal const * geneticArchitecture;
	const static int K = nCaste * nTrait * ageMax; // define K
};


//definition of the class individual
class Male;
class Individual {
	friend class Male; //male is friend class of individuals. 
public:
	enum Traits { survival };// here we only use one trait the age specific surviva probabilities
	void incrementAge() { ++age; }   // member function to increase the age of an indididual 
	void extrinsicMortality(double deathRate) { extrinsicSurvival = 1.0 - deathRate; }// extrinsic mortality function 
	bool doesSurvive() const { return age < ageMax && rnd::bernoulli(extrinsicSurvival * lifeHistory(survival, age)); } // function that returdns the age specific survival probability for an individual (extrinsic and intrindic mortality included)
    friend std::ostream& operator<< (std::ostream&, const Individual&); 
	double getLifeHistoryTrait(const Traits &tr) const { return lifeHistory(static_cast<Matrix::size_type>(tr), age); }
	const Matrix& getLifeHistory() const { return lifeHistory; }//returnd individual life history
	int getAge() const { return age; }//returns the individuals age
	void getData(std::vector<double>&) const;//retuns age specific survival probability
protected:
    Individual() : lifeHistory(nTrait, ageMax), age(0) {}//constructor
	Individual(const Individual&) = default;//alternative constructor
	std::array<Gene, nGene> genome;
	int age;
	double extrinsicSurvival;
    Matrix lifeHistory;
	virtual void develop() = 0; 
	void mutate(); //function prototype
};


////definition of the class female
class Queen;
class Female : public Individual { //inherit properties from the class individual
	friend class Queen;
public:
	Female() = default;//constructor
	Female(const Female&) = delete;//constructor
	Female(Queen const * const mother);//constructor
	~Female() { ++ageOfDeath[age]; } //destructor; adds the death to the age of death vector 
	static std::array<size_t, ageMax + 1u> ageOfDeath;//static vector to store the ages at death 
private:
	void develop();
};

////definition of the class male
class Male {
	friend class Female;
public:
	Male(Individual const * const);//constructor
	Male(const Male&) = default;//constructor
private:
	Male() = delete;
	void mutate();
	std::array<Gene, nGene / 2> genome;//haploid genome
};

////definition of the class queen
class Queen: public Individual {//inherit properties from class individual
public:
	friend class Female;   
	Queen() = delete;//  constructor
	Queen(Female const * const, const Male&);//  constructor
	~Queen() { ++ageOfDeath[age]; } //destructor; adds one at the given age class
	static std::array<size_t, ageMax + 1u> ageOfDeath; ////static vector to store the ages at death 
private:
	Male sperm;
	void develop();
};

#endif