
/*=============================================================================================================
parameters.h
===============================================================================================================

file to pass parameters to the simulation

 C++-code accompanying: 
 
        authors : Boris H. Kramer, G. Sander van Doorn, Babak M. S. Arani, Ido Pen
        title : Eusociality and the evolution of aging in superorganisms
 

 Written by:
        Boris H. Kramer and G. Sander van Doorn
        Groningen Institute for Evolutionary Life Sciences (Gelifes)
        University of Groningen
        the Netherlands
 
 Program version
        01/03/2019	

=============================================================================================================*/


#ifndef parameters_h   // checks if parameters are already included
#define parameters_h

//colony dynamics
const int nColony = 10;													//set the number of colonies in the simulation
const int tEnd = 200;													//set simulation time steps	
const int dataInterval = 10;											//save date every dataInterval time steps 

//life history parameters
const int ageMax = 20;												    //set number of age classes , maximum lifespan
const double foragingRisk	= 0.0;										//set the level of extrinsic mortality for individual that leave the colony for foraging
const double survivalOffset	= 0.7;										//set initial survival probability for all age classes and castes, choose values from -1:1 
const double productivity	= 1.0;										//set amount of resources obtained per foraging trip 
const double queenFecundity = 10.0;										//set queen fecundity (higher setting of fecundity makes eggs cheaper; more eggs per resource unit) 
const double workerFecundity = 0.0;										//set worker fecundity (higher setting of fecundity makes eggs cheaper ( more eggs per resource unit) 

// Set scenario
const bool gyneReplacesQueen = true;									//toggle monogyny (false) / polygyny (true)
const bool workerReplacesQueen = true;									//set worker replacement (worker can take over the colony after queen death if set to true)

//genetics
const int nGene = 40;													//set number of genes, has to be even more genes lead to a larger target for mutations (each gene has an effect on all age classes and the effect is summed up (ind. cpp l. 170?)
const int nCaste = 2;													//set number of castes (keep at 2) 
const int nTrait = 1;													//sset number of traits (keep at 1) 
const double mutationRate = 1.0e-4;										//rate of mutation per gene per generation
const double sigma[nCaste][nTrait] = { { 0.08 },{ 0.08 } };				//sets mutational effect size over age classes, increase = more age classes are affected by a mutation   ; affects standard deviations of the multivariate normal and eigenvectors of the mutational variance covariance matrix 
const double alpha[nTrait] = {0.0};										//sets correlation between subsequent age classes within trait  
const double beta[nTrait][(nCaste * (nCaste - 1)) / 2] = { { 0.0 } };	//genetic correlation within trait, between castes
const double eta[nTrait] = {-0.2};										//set mutation bias (relative to mutational effect size);  the final value is -.02*the standart derivation of the mutational effect size

#endif